package net.krecan.demo.bank;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.BatchSize;

@Entity
@Table(name="client")
public class Client  {
	
	@Id @GeneratedValue(strategy=GenerationType.AUTO)
	private Integer id;

	@Column
	private Long personId;
	
	@Column
	private String name;
	
	@OneToMany(cascade=CascadeType.ALL)
	@JoinColumn(name="client_id", nullable=false)
	@BatchSize(size=10)
	private Set<Account> accounts = new HashSet<Account>();
	
	@OneToMany(cascade=CascadeType.ALL)
	@JoinColumn(name="client_id", nullable=false)
	@BatchSize(size=10)
	private Set<Address> addresses = new HashSet<Address>();
	
	
	protected Client() {
		super();
	}

	public Client(Long personId) {
		super();
		this.personId = personId;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public void addAddress(Address address)
	{
		addresses.add(address);
	}
	
	public void removeAddress(Address address)
	{
		addresses.remove(address);
	}
	
	public void addAccount(Account account)
	{
		accounts.add(account);
	}
	
	public void removeAccount(Account account)
	{
		accounts.remove(account);
	}

	public Long getPersonId() {
		return personId;
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((personId == null) ? 0 : personId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Client other = (Client) obj;
		if (personId == null) {
			if (other.personId != null)
				return false;
		} else if (!personId.equals(other.personId))
			return false;
		return true;
	}

	public Set<Account> getAccounts() {
		return Collections.unmodifiableSet(accounts);
	}

	public Set<Address> getAddresses() {
		return Collections.unmodifiableSet(addresses);
	}

	/**
	 * Constructs a <code>String</code> with all attributes
	 * in name = value format.
	 *
	 * @return a <code>String</code> representation 
	 * of this object.
	 */
	public String toString()
	{
	    final String TAB = "    ";
	    
	    String retValue = "";
	    
	    retValue = "Client ( "
	        + super.toString() + TAB
	        + "id = " + this.id + TAB
	        + "personId = " + this.personId + TAB
	        + "name = " + this.name + TAB
	        + "accounts = " + this.accounts + TAB
	        + "addresses = " + this.addresses + TAB
	        + " )";
	    
	    return retValue;
	}
	

}
